// HertzEdit.cpp: Implementierungsdatei
//
// Autor: Olaf Groeger
// EMail: Olaf.Groeger@gmx.de
//
// Status: Macht damit, was ihr wollt
//
////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
// #include "gaspneu.h"
#include "HertzEdit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CHertzEdit

CHertzEdit::CHertzEdit()
{
	m_dValue=0.0;
	m_bNegativAllowed=TRUE;
	m_strValue= _T("");
}

CHertzEdit::CHertzEdit(BOOL bNegativAllowed)
{
	m_bNegativAllowed=bNegativAllowed;
	m_dValue=0.0;
	m_strValue= _T("");
}


CHertzEdit::~CHertzEdit()
{
}


BEGIN_MESSAGE_MAP(CHertzEdit, CEdit)
	//{{AFX_MSG_MAP(CHertzEdit)
	ON_WM_CHAR()
	ON_WM_KILLFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CHertzEdit 

double CHertzEdit::CStringToDouble(CString &str)
{
	double d;
	
	if (((m_bNegativAllowed==FALSE) && (str.FindOneOf("-") != -1)) ||
		(str.CompareNoCase("invalid") == 0))
	{
		d=-1.0;
	}
	else
	{
		int	i=str.GetLength();
		std::istringstream ist(str.GetBuffer(i));
		str.ReleaseBuffer();
		double d1;
		char c[3];
		ist >> d1 >> c;
		CString strEinheit(c);
		if (!strEinheit.CompareNoCase("Hz"))
		{
			d=d1;
		}
		else if (!strEinheit.CompareNoCase("kHz"))
		{
			d=d1*1000.0;
		}
		else if (!strEinheit.CompareNoCase("MHz"))
		{
			d=d1*1000000.0;
		}
	}
	return(d);
}

std::string CHertzEdit::DoubleToString(double d)
{
	double d1, d2;

	std::ostringstream ost;
	ost.setf(std::ios_base::fixed, std::ios_base::floatfield);

	d2=fabs(d);

	if (d < 0.0 && !m_bNegativAllowed)
	{
		ost << "Invalid" << std::ends;
	}
	else if (d2 < 1000.0)	// Hz-Bereich
	{		
		d1 = d;
		ost << std::setprecision(1);
		ost << d1;
		StripEndingZero(ost);
		ost << " Hz" << std::ends;
	}
	else if (d2 < 1000000.0)	// Khz-Bereich
	{
		d1 = d / 1000.0;
		ost << std::setprecision(4);
		ost << d1;
		StripEndingZero(ost);
		ost << " KHz" << std::ends;
	}
	else if (d2 > 10.0e9)		// zu gro fuer format::fixed
	{
		ost << "Invalid" << std::ends;
	}
	else						// MHz-Bereich
	{
		d1 = d / 1000000.0;
		ost << std::setprecision(7);
		ost << d1;
		StripEndingZero(ost);
		ost << " MHz" << std::ends;
	}
	return(ost.str());

}

void CHertzEdit::StripEndingZero(std::ostringstream &oststr)
{
	std::string strString= oststr.str();
	std::string::iterator it = strString.end();
	--it;
	while ( *(it) == '0' && it != strString.begin()) // Letztes Zeichen == '0' ?
	{										// ja
		if ( *(it-1) != '.')				// ist davor der '.' ?
		{									// nein => weg damit
			strString.erase(it);
			it = strString.end();
			--it;
		}
		else
		{
			break;							// ja => '0' dranlassen (z.B. 1.0)
		}
	}												// nein => nichts machen
	oststr.seekp(0);				// oststr neu initialisieren
	oststr << strString;
}

void CHertzEdit::SetValue(double dValue)
{
	m_dValue=dValue;
	m_strValue=DoubleToString(m_dValue).c_str();
}

double CHertzEdit::GetValue()
{
	return(m_dValue);
}

CHertzEdit& CHertzEdit::operator =(double dValue)
{
	SetValue(dValue);
	return(*this);
}

BOOL CHertzEdit::GetValue(HWND hWndCtrl)
{
	return(GetValueFromWindow(hWndCtrl));
}

BOOL CHertzEdit::SetValue(HWND hWndCtrl)
{
	m_strValue = DoubleToString(m_dValue).c_str();
	SetValueToWindow(hWndCtrl);
	return(m_strValue.CompareNoCase("invalid") == 0);
}

void CHertzEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Code fr die Behandlungsroutine fr Nachrichten hier einfgen und/oder Standard aufrufen
	
	if (iscntrl(nChar))
	{
		CEdit::OnChar(nChar, nRepCnt, nFlags);
		return;
	}
	
	CString str_nChar(nChar);
	CString strZiel;
	if (m_bNegativAllowed)
		strZiel=str_nChar.SpanIncluding("0123456789kKmMhHzZ.-");
	else
		strZiel=str_nChar.SpanIncluding("0123456789kKmMhHzZ.");

	if (strZiel.IsEmpty())
		return;
	
	strZiel=str_nChar.SpanIncluding(".kKmMhHzZ");
	if (strZiel.IsEmpty()) {					// Zeichen ?, ja dann
		CEdit::OnChar(nChar, nRepCnt, nFlags);	// Basisklasse
		return;
	}

	int nTextLen=GetWindowTextLength();
	char* pChar=new char[nTextLen+1];		// incl. abschliessender \0
	GetWindowText(pChar, nTextLen+1);
	CString strText(pChar);

	delete[] pChar;

	if (nChar=='.') {
		if (strText.Find('.')== -1)	{			// ist schon ein Punkt da
			CEdit::OnChar(nChar, nRepCnt, nFlags); // nein -> Basisklasse
			return;
		} else {
			return;							// ja -> Abgang
		}
	}
	
	BOOL wasK, wasM, wasHz;

	if (strText.Remove('m') || strText.Remove('M')) wasM=TRUE; else wasM=FALSE;
	if (strText.Remove('k') || strText.Remove('K')) wasK=TRUE; else wasK=FALSE;
	if (strText.Remove('h') || strText.Remove('H')) wasHz=TRUE; else wasHz=FALSE;
	if ((strText.Remove('z') || strText.Remove('Z')) && wasHz)
		wasHz=TRUE;
	else
		wasHz=FALSE;

	strText.Remove(' ');

	if (nChar=='k' || nChar=='K') {
		strText+=" KHz";
		SetWindowText(strText);
		SetModify();
		return;
	}
	if (nChar=='m' || nChar=='M') {
		strText+=" MHz";
		SetWindowText(strText);
		SetModify();
		return;
	}
	if (nChar=='h' || nChar=='H' || nChar=='z' || nChar=='Z') {
		if (wasK) {
			strText+=" KHz";
			SetWindowText(strText);
			SetModify();
			return;
		}
		if (wasM) {
			strText+=" MHz";
			SetWindowText(strText);
			SetModify();
			return;
		}
		strText+=" Hz";
		SetWindowText(strText);
		SetModify();
		return;
	}
	ASSERT(FALSE); //CEdit::OnChar(nChar, nRepCnt, nFlags);}
}

////////////////////////////////////////////////////////////////////////
void AFXAPI DDX_HertzEdit(CDataExchange* pDX, int nIDC, CHertzEdit& her)
{
	HWND hWndCtrl=pDX->PrepareEditCtrl(nIDC);

	if (pDX->m_bSaveAndValidate)
	{
		her.GetValue(hWndCtrl);
	}
	else
	{
		her.SetValue(hWndCtrl);
	}
}

CHertzEdit::CHertzEdit(const CHertzEdit& he)
{
	m_bNegativAllowed=he.m_bNegativAllowed;
	m_dValue=he.m_dValue;
	m_strValue=he.m_strValue;
}

void CHertzEdit::SetValueToWindow()
{
	HWND hWnd = GetSafeHwnd();
	SetValueToWindow(hWnd);
}

BOOL CHertzEdit::GetValueFromWindow()
{
	HWND hWnd = GetSafeHwnd();
	return(GetValueFromWindow(hWnd));
}

void CHertzEdit::OnKillFocus(CWnd* pNewWnd) 
{
	CEdit::OnKillFocus(pNewWnd);
	
	GetValueFromWindow();
	SetValueToWindow();
}

void CHertzEdit::SetValueToWindow(HWND hWnd)
{
	if (m_hWnd)
		::SetWindowText(hWnd, m_strValue);
}

BOOL CHertzEdit::GetValueFromWindow(HWND hWnd)
{
	CString strValue;
	TCHAR*	pChar;
	BOOL	bReturn;
	int	i;
	double d;

	i= ::GetWindowTextLength(hWnd);
	if (!i) return(FALSE);

	pChar=new char[i+1];
	::GetWindowText(hWnd, pChar, i+1);
	strValue=pChar;
	delete[] pChar;

	d=CStringToDouble(strValue);

	if ((d < 0.0) && !m_bNegativAllowed)
	{ // Inhalt ist ungltig
		if (m_dValue > 0.0)
			m_dValue=-1.0;		// Wert ungltig machen
		bReturn=FALSE;		// sonst ungltigen Wert belassen
	}
	else
	{
		m_dValue = d;
		bReturn = TRUE;
	}
	m_strValue = DoubleToString(m_dValue).c_str();
	return(bReturn);
}
